@@ -9,17 +9,21 @@ from os.path import dirname, abspath
 root = dirname(dirname(abspath(__file__)))
 patsy_ref = root + "/doc/API-reference.rst"
 
-doc_re = re.compile("^\.\. (.*):: ([^\(]*)")
+doc_re = re.compile(r"^\.\. (.*):: ([^\(]*)")
+
+
 def _documented(rst_path):
     documented = set()
-    for line in open(rst_path):
-        match = doc_re.match(line.rstrip())
-        if match:
-            directive = match.group(1)
-            symbol = match.group(2)
-            if directive not in ["module", "ipython"]:
-                documented.add(symbol)
-    return documented
+    with open(rst_path) as rst_file:
+        for line in rst_file:
+            match = doc_re.match(line.rstrip())
+            if match:
+                directive = match.group(1)
+                symbol = match.group(2)
+                if directive not in ["module", "ipython"]:
+                    documented.add(symbol)
+        return documented
+
 
 try:
     import patsy
@@ -28,7 +32,7 @@ except ImportError:
     import patsy
 
 documented = set(_documented(patsy_ref))
-#print(documented)
+# print(documented)
 exported = set(patsy.__all__)
 missed = exported.difference(documented)
 extra = documented.difference(exported)
